import madmike.util.Log;


import javax.swing.JFrame;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiUnavailableException;

import java.io.File;
import java.io.IOException;


public class AdagioGUI extends JFrame implements ActionListener, ItemListener {
    private static final String OUTFILE_NAME = "Last.mid";

    
    private JComboBox<String> cmbMidiDevice = new JComboBox<String>();
    private JLabel lblRuleSet = new JLabel("*");
    private JCheckBox chkRuleSkipStepwise = new JCheckBox("Skip-stepwise motion", false);
    private JCheckBox chkRuleForbidThreeNoteRepeat = new JCheckBox("Forbid three-note repeat", false);
    private JCheckBox chkRuleInitialTonic = new JCheckBox("Initial tonic", false);
    private JCheckBox chkRuleCadence = new JCheckBox("Cadence rule", false);
    private JCheckBox chkRuleTritoneResolution = new JCheckBox("Tritone resolution", false);
    private JCheckBox chkRuleOctaveRange = new JCheckBox("Octave range rule", false);
    private JCheckBox chkRuleHarmonic = new JCheckBox("Harmonic rule", false);
    private JCheckBox chkRuleMelodic = new JCheckBox("Melodic rule", false);
    private JCheckBox chkRuleOneStepwise = new JCheckBox("One voice stepwise", false);
    private JCheckBox chkRuleForbidParallelPerfects = new JCheckBox("Forbid parallel perfects", false);
    private JCheckBox chkRuleForbidParallelFourths = new JCheckBox("Forbid parallel fourths", false);
    private JCheckBox chkRuleForbid64Chords = new JCheckBox("Forbid 6-4 chords", false);
    private JCheckBox chkRuleForbidClimaxRepeat = new JCheckBox("Forbid climax repeat", false);
    private JComboBox<String> cmbRuleSet = new JComboBox<String>(new String[] {"A", "B", "C", "D", "E", "F", "G", "H"});
    private JButton btnComposeScore = new JButton("Compose One Section");
    private JButton btnComposeMovement = new JButton("Compose One Movement");
    private JButton btnStopPlayback = new JButton("Stop");

    private boolean cmbMidiDeviceHasChanged = false;

    private Musician musician = new Musician();
    
    private PlaybackThread playbackThread;

    
    public AdagioGUI() {
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	setSize(480, 640);
	setTitle("Experiment Two: Adagio (V" + Engineer.APP_VERSION + ")");

	int componentWidth = 200;
	int componentHeight = 40;

	Box box = new Box(BoxLayout.Y_AXIS);
	
	box.add(Box.createVerticalGlue());
	// box.add(Box.createVerticalStrut(32));

	// Combo Box for MIDI device
	String[] devNames = null;
	try {
	    devNames = Musician.getReceiverDeviceNames();
	}
	catch(MidiUnavailableException ex) {
	    Log.write(Log.WARNING, "MIDI unavailable.");
	}
	for(int i=0; i<devNames.length; ++i)
	    cmbMidiDevice.addItem(devNames[i]);
	String devStr = "";
	try {
	    devStr = Engineer.getConfString("MIDIDEVICE");
	    for(int i=0; i<devNames.length; ++i) {
		if(devStr.equals(devNames[i])) {
		    cmbMidiDevice.setSelectedIndex(i);
		    break;
		}
	    }
	}
	catch(Exception ex) {
	    Log.write(Log.WARNING, "Could not get MIDI device from configuration file.");
	}
	cmbMidiDevice.addItemListener(this);
	cmbMidiDevice.setAlignmentX(Component.CENTER_ALIGNMENT);
	cmbMidiDevice.setMaximumSize(new Dimension(componentWidth, componentHeight));
	box.add(cmbMidiDevice);

	box.add(Box.createVerticalStrut(32));
	
	// Label for rule set
	lblRuleSet.setFont(lblRuleSet.getFont().deriveFont(32.0f));
	lblRuleSet.setAlignmentX(Component.CENTER_ALIGNMENT);
	box.add(lblRuleSet);

	box.add(Box.createVerticalStrut(24));

	// Checkboxes for rules: two columns
	Box boxRulesContainer = new Box(BoxLayout.X_AXIS);
	Box boxRules1 = new Box(BoxLayout.Y_AXIS);
	boxRules1.setAlignmentY(Component.TOP_ALIGNMENT);
	// boxRules1.setOpaque(true);  // for debugging
	// boxRules1.setBackground(java.awt.Color.RED);  // for debugging
	/*
	javax.swing.UIManager.put("CheckBox.disabledText", java.awt.Color.BLACK);
	javax.swing.UIManager.put("CheckBox.disabledIcon", javax.swing.UIManager.get("CheckBox.selectedIcon"));
	*/
	javax.swing.SwingUtilities.updateComponentTreeUI(chkRuleSkipStepwise);
	chkRuleSkipStepwise.setEnabled(false);
	boxRules1.add(chkRuleSkipStepwise);
	chkRuleForbidThreeNoteRepeat.setEnabled(false);
	boxRules1.add(chkRuleForbidThreeNoteRepeat);
	chkRuleInitialTonic.setEnabled(false);
	boxRules1.add(chkRuleInitialTonic);
	chkRuleCadence.setEnabled(false);
	boxRules1.add(chkRuleCadence);
	chkRuleTritoneResolution.setEnabled(false);
	boxRules1.add(chkRuleTritoneResolution);
	chkRuleOctaveRange.setEnabled(false);
	boxRules1.add(chkRuleOctaveRange);
	chkRuleHarmonic.setEnabled(false);
	boxRules1.add(chkRuleHarmonic);
	Box boxRules2 = new Box(BoxLayout.Y_AXIS);
	boxRules2.setAlignmentY(Component.TOP_ALIGNMENT);
	// boxRules2.setOpaque(true);  // for debugging
	// boxRules2.setBackground(java.awt.Color.BLUE);  // for debugging
	chkRuleMelodic.setEnabled(false);
	boxRules2.add(chkRuleMelodic);
	chkRuleOneStepwise.setEnabled(false);
	boxRules2.add(chkRuleOneStepwise);
	chkRuleForbidParallelPerfects.setEnabled(false);
	boxRules2.add(chkRuleForbidParallelPerfects);
	chkRuleForbidParallelFourths.setEnabled(false);
	boxRules2.add(chkRuleForbidParallelFourths);
	chkRuleForbid64Chords.setEnabled(false);
	boxRules2.add(chkRuleForbid64Chords);
	chkRuleForbidClimaxRepeat.setEnabled(false);
	boxRules2.add(chkRuleForbidClimaxRepeat);
	boxRulesContainer.add(boxRules1);
	boxRulesContainer.add(Box.createHorizontalStrut(24));
	boxRulesContainer.add(boxRules2);
	box.add(boxRulesContainer);

	box.add(Box.createVerticalStrut(32));

	// Button for composing a complete movement
	btnComposeMovement.addActionListener(this);
	btnComposeMovement.setAlignmentX(Component.CENTER_ALIGNMENT);
	btnComposeMovement.setPreferredSize(new Dimension(componentWidth, componentHeight));
	btnComposeMovement.setMaximumSize(new Dimension(componentWidth, componentHeight));
	box.add(btnComposeMovement);

	box.add(Box.createVerticalStrut(32));

	// Combo Box for Rule Set
	cmbRuleSet.setAlignmentX(Component.CENTER_ALIGNMENT);
	cmbRuleSet.setMaximumSize(new Dimension(componentWidth, componentHeight));
	box.add(cmbRuleSet);

	box.add(Box.createVerticalStrut(12));

	// Button for composing a complete movement
	btnComposeScore.addActionListener(this);
	btnComposeScore.setAlignmentX(Component.CENTER_ALIGNMENT);
	box.add(btnComposeScore);
	btnComposeScore.setPreferredSize(new Dimension(componentWidth, componentHeight));
	btnComposeScore.setMaximumSize(new Dimension(componentWidth, componentHeight));

	box.add(Box.createVerticalStrut(32));

	// Button for stopping playback
	btnStopPlayback.addActionListener(this);
	btnStopPlayback.setAlignmentX(Component.CENTER_ALIGNMENT);
	btnStopPlayback.setPreferredSize(new Dimension(componentWidth, componentHeight));
	btnStopPlayback.setMaximumSize(new Dimension(componentWidth, componentHeight));
	box.add(btnStopPlayback);

	box.add(Box.createVerticalGlue());

	getContentPane().add(box);

	// On shutdown
	Runtime.getRuntime().addShutdownHook(new Thread() {
		public void run() {
		    if(cmbMidiDeviceHasChanged) {
			try {
			    Engineer.writeConfString("MIDIDEVICE", cmbMidiDevice.getSelectedItem().toString());
			}
			catch(IOException ex) {
			    Log.write(Log.WARNING, "Exception writing MIDI device to configuration file.");
			}
		    }
		}
	    });
    }


    public Score[] composeMovement() {
	String[] ruleSets = {"A", "B", "C", "D", "E", "F", "G", "H", "H", "H", "H", "H"};
	int[] lengths = {12, 12, 12, 12, 12, 12, 12, 12, 2, 2, 2, 2};
	int numScores = ruleSets.length;
	Score[] scores = new Score[numScores];

	Composer co = new Composer();
	
	for(int i=0; i<numScores; ++i) {
	    scores[i] = new Score(4, lengths[i]);
	    co.compose(scores[i], ruleSets[i], "RandomNumberGenerator", null);
	}

	return scores;
    }

    
    private void writeMidiFile(Score[] scores, File outfile) {
	Log.write(Log.INFORMATION, "Writing score(s) to MIDI file " + OUTFILE_NAME + "...");
	try {
	    Transcriber tr = new Transcriber(scores[0].getNumVoices());
	    for(int i=0; i<scores.length; ++i) {
		String ruleSetStr = scores[i].getRuleSetStr();
		tr.addScore(scores[i], ruleSetStr != null ? "Rule Set " + ruleSetStr : null);
	    }
	    tr.writeMidiFile(outfile);
	}
	catch(Exception ex) {
	    Log.write(Log.WARNING, "Exception writing score(s) to MIDI file: " + ex.toString() + "\n");
	}
    }
    

    public void actionPerformed(ActionEvent ev) {
	if(ev.getSource() == btnComposeMovement) {
	    Log.write(Log.DEBUG, "Compose Movement button pressed...");

	    // Stop musician, in case it's playing
	    if(playbackThread != null) {
		playbackThread.stopThread();
	    }
	    
	    Score[] scs = composeMovement();

	    Log.write(Log.INFORMATION, "Composed " + scs.length + " scores.");

	    // Write scores to MIDI file
	    writeMidiFile(scs, new File(OUTFILE_NAME));
	    
	    playbackThread = new PlaybackThread(scs);
	    playbackThread.start();
	}  // END btnComposeMovement
	else if(ev.getSource() == btnComposeScore) {
	    Log.write(Log.DEBUG, "Compose Section button pressed...");

	    // Stop musician, in case it's playing
	    if(playbackThread != null) {
		playbackThread.stopThread();
	    }
	    
	    int length = 12;
	    String ruleSetStr = cmbRuleSet.getSelectedItem().toString();

	    Score score = new Score(4, length);
	    Composer co = new Composer();
	    
	    co.compose(score, ruleSetStr, "RandomNumberGenerator", null);

	    // Write score to MIDI file
	    writeMidiFile(new Score[] {score}, new File(OUTFILE_NAME));
	    
	    playbackThread = new PlaybackThread(new Score[] {score});
	    playbackThread.start();
	}  // END btnComposeScore
	else if(ev.getSource() == btnStopPlayback) {
	    Log.write(Log.DEBUG, "Stop button pressed...");
	    
	    if(playbackThread != null) {
		playbackThread.stopThread();
	    }
	}  // END btnStopPlayback
    }  // END actionPerformed


    public void itemStateChanged(ItemEvent ev) {
	if(ev.getSource() == cmbMidiDevice) {
	    cmbMidiDeviceHasChanged = true;
	}  // END cmbMidiDevice
    }


    /* --- Main Program --- */
    public static void main(String[] args) {
	Log.setLevel(Log.INFORMATION);
	
	AdagioGUI frmMain = new AdagioGUI();
	// frmMain.pack();
	frmMain.setVisible(true);
    }


    /**
     * Inner class: Thread controlling playback by the Musician.
     */
    private class PlaybackThread extends Thread {
	private volatile boolean running = false;

	private Score[] scores;

	public PlaybackThread(Score[] scs) {
	    scores = scs;
	}

	public void run() {
	    running = true;

	    Log.write(Log.INFORMATION, "Playback thread started.");

	    // Set musician's MIDI device
	    try {
		MidiDevice.Info[] infos = MidiSystem.getMidiDeviceInfo();
		String selected = cmbMidiDevice.getSelectedItem().toString();
		int deviceIndex = 0;
		for(int i=0; i<infos.length; ++i) {
		    if(selected.equals(infos[i].getName())) {
			deviceIndex = i;
			break;
		    }
		}
		musician.setMidiDevice(MidiSystem.getMidiDevice(infos[deviceIndex]));

		musician.open();
	    }
	    catch(Exception ex) {
		Log.write(Log.WARNING, "Exception setting musician's MIDI device.");
	    }

	    // Play scores
	    try {
		for(int i=0; i<scores.length && running; ++i) {
		    String ruleSetStr = scores[i].getRuleSetStr();
		    Log.write(Log.INFORMATION, "Playing score #" + i + (ruleSetStr != null ? ", Rule Set: " + ruleSetStr : "") + " ...");

		    // Display rule set
		    lblRuleSet.setText(ruleSetStr);
		    chkRuleSkipStepwise.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_SKIPSTEPWISE));
		    chkRuleForbidThreeNoteRepeat.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_FORBIDTHREENOTEREPEAT));
		    chkRuleInitialTonic.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_INITIALTONIC));
		    chkRuleCadence.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_CADENCE));
		    chkRuleTritoneResolution.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_TRITONERESOLUTION));
		    chkRuleOctaveRange.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_OCTAVERANGE));
		    chkRuleHarmonic.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_HARMONIC));
		    chkRuleMelodic.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_MELODIC));
		    chkRuleOneStepwise.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_ONE_STEPWISE));
		    chkRuleForbidParallelPerfects.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_FORBIDPARALLELPERFECTS));
		    chkRuleForbidParallelFourths.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_FORBIDPARALLELFOURTHS));
		    chkRuleForbid64Chords.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_FORBID64CHORDS));
		    chkRuleForbidClimaxRepeat.setSelected(Ruleset.getRuleFromSet(ruleSetStr, Ruleset.RULE_FORBIDCLIMAXREPEAT));

		    // Play!
		    musician.play(scores[i]);
		}
	    }
	    catch(Exception ex) {
		Log.write(Log.WARNING, "Exception playing score(s): " + ex.toString() + "\n");
	    }

	    musician.close();

	    Log.write(Log.INFORMATION, "Playback thread finished.");

	    running = false;
	}  // 

	/**
	 * Be careful: This does NOT get called from the PlaybackThread,
	 * but from some controlling thread!
	 */
	public void stopThread() {
	    Log.write(Log.INFORMATION, "Stopping playback thread...");

	    running = false;
	    interrupt();

	    try {
		Log.write(Log.DEBUG, "Stopping musician...");
		musician.stopPlayback();
		musician.close();
		Log.write(Log.DEBUG, "Musician stopped.");
	    }
	    catch(Exception ex) {
		Log.write(Log.WARNING, "Exception stopping musician in AdagioGUI.stopMusician().");
	    }
	}


	public boolean isRunning() {return running;}
    }
}
