import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;

import madmike.util.Log;


public class FileNumberGenerator implements Inspiration
{
    File file;
    boolean rememberPosition = true;
    private RandomAccessFile reader;

    public void openFile (File f)
    {
	file = f;
	try
	{
	    Log.write (Log.INFORMATION, "File note generator started for file '" + f.getName () + "'...");
	    reader = new RandomAccessFile (f, "r");
	    if (rememberPosition && new File (f.getName () + ".pos").exists ())
	    {
		BufferedReader posFile = new BufferedReader (new FileReader ((f.getName () + ".pos")));
		long filePointer = Long.parseLong (posFile.readLine ());
		Log.write (Log.INFORMATION, "Seeking to position " + filePointer + "...");
		reader.seek (filePointer);
	    }
	}
	catch (IOException ex)
	{
	    Log.write (Log.ERROR, "IOException: " + ex.toString ());
	    System.exit (255);
	}
    }

    public void setParms (String [] parms)
    {
	String fileName = null;
	if (parms == null)
	{
	    Log.write (Log.ERROR, "FileNumberGenerator needs parameters: a file name, and an optional 'NOPOSITION'.");
	    System.exit (255);
	}
	for (int i=0; i<parms.length; i++)
	{
	    if (parms [i].toUpperCase ().equals ("NOPOSITION"))
		rememberPosition = false;
	    else fileName = parms [i];
	}

	if (fileName == null)
	{
	    Log.write (Log.ERROR, "No file name specified for FileNumberGenerator!");
	    System.exit (255);
	}
	else openFile (new File (fileName));
    }

    public int generateValue (int sup)
    {
	try
	{
	    int ch;
	    byte [] chars = new byte [2];
	    for (int i=0; i<2; i++)
	    {
		if ((ch = reader.read ()) == -1)
		{
		    Log.write (Log.INFORMATION, "End of file reached, resetting stream.");
		    reader.seek (0);
		    ch = reader.read ();
		}
		chars [i] = (byte) ch;
	    }
	    // ch = Character.getNumericValue (cbuf [0]);
	    ch = Integer.parseInt (new String (chars));
	    // Log.write (Log.INFORMATION, "" + ch);
	    if (ch >= sup) ch %= sup;
	    return ch;
	}
	catch (IOException ex)
	{
	    Log.write (Log.ERROR, "IOException: " + ex.toString ());
	    System.exit (255);
	}
	return 0;  // Unreachable statement, but the compiler wants it.
    }

    public void dispose ()
    {
	if (rememberPosition)
	    try
	    {
		Log.write (Log.INFORMATION, "Writing current position = " + reader.getFilePointer () + " to file...");
		FileWriter posFile = new FileWriter ((file.getName () + ".pos"));
		posFile.write (Long.toString (reader.getFilePointer ()));
		posFile.close ();
	    }
	    catch (IOException ex)
	    {
		Log.write (Log.WARNING, "Cannot write position file.");
	    }

	try {reader.close ();}
	catch (IOException ex)
	{
	    Log.write (Log.ERROR, "IOException: " + ex.toString ());
	    System.exit (255);
	}
    }
}
