import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.util.StringTokenizer;
import java.util.Vector;


public class Librarian
{
    public static Score fetch (File f) throws FileNotFoundException, IOException
    {
	BufferedReader reader = new BufferedReader (new java.io.FileReader (f));

	Vector<String> voiceLines = new Vector<String> ();
	Vector<String> dynLines = new Vector<String> ();

	// For the meta information
	String ruleSetStr = null;
	String infoStr = null;

	String line;
	while ((line = reader.readLine ()) != null) {
	    // Voice lines are not empty, they do not contain an '=' sign,
	    // and they start with possibly white space, and a 'V'.
	    if (!line.trim ().equals ("") && line.indexOf("=") == -1 && line.trim ().charAt (0) == 'V')
		voiceLines.addElement (line);
	    else if(line.toUpperCase().startsWith("RULESET")) {
		int eqIndex = line.indexOf("=");
		ruleSetStr = line.substring(eqIndex + 1).trim();
	    }
	    else if(line.toUpperCase().startsWith("INFO")) {
		int eqIndex = line.indexOf("=");
		infoStr = line.substring(eqIndex + 1).trim();
	    }
	    // Dynamics lines start with "DYN<n>" for voice number n
	    if(line.trim().toUpperCase().startsWith("DYN")) {
		dynLines.addElement(line);
	    }
	}

	reader.close ();

	int [] [] notes = new int [voiceLines.size ()] [];

	// Note (no pun intended!): Voices are in reverse order here, i.e.
	// V0 is on the bottom of the file.
	for (int j=0; j<voiceLines.size (); j++)
	{
	    String token;
	    StringTokenizer tok = new StringTokenizer (voiceLines.elementAt (j), " ");
	    notes [j] = new int [tok.countTokens () - 1];
	    token = tok.nextToken ();
	    for (int i=0; tok.hasMoreTokens (); i++)
		notes [j] [i] = nameToHINote (tok.nextToken ());
	}

	Score sc = new Score (notes.length, notes [0].length);

	// Set notes
	for (int j=0; j<notes.length; j++)
	    for (int i=0; i<notes [0].length; i++)
		sc.setNote (notes.length - j - 1, i, notes [j] [i]);

	// Set dynamics
	for(int i=0; i<dynLines.size(); ++i) {
	    String dynLine = dynLines.elementAt(i);
	    int eqIndex = dynLine.indexOf("=");
	    int voice = Integer.parseInt(dynLine.substring(3, eqIndex).trim());
	    int dyn = Integer.parseInt(dynLine.substring(eqIndex + 1).trim());
	    Log.write(Log.DEBUG, "Setting dynamics for voice " + voice + " to " + dyn + ".");
	    sc.setDynamics(voice, dyn);
	}

	if(ruleSetStr != null)
	    sc.setRuleSetStr(ruleSetStr);
	if(infoStr != null)
	    sc.setInfo(infoStr);

	return sc;
    }

    public static void store (Score sc, File f) throws IOException
    {
	BufferedWriter writer = new BufferedWriter (new java.io.FileWriter (f));

	String ruleSetStr = sc.getRuleSetStr();
	if(ruleSetStr != null)
	    writer.write("RULESET=" + ruleSetStr + "\n");
	
	String info = sc.getInfo();
	if(info != null)
	    writer.write("INFO=" + info + "\n");
	
	writer.write (sc.toSimpleNotation ());
	writer.flush ();
	writer.close ();
    }


    public static int nameToHINote (String n)
    {
	if (n.equals ("C-0")) return 0;
	if (n.equals ("D-0")) return 1;
	if (n.equals ("E-0")) return 2;
	if (n.equals ("F-0")) return 3;
	if (n.equals ("G-0")) return 4;
	if (n.equals ("A-0")) return 5;
	if (n.equals ("B-0")) return 6;
	if (n.equals ("C-1")) return 7;
	if (n.equals ("D-1")) return 8;
	if (n.equals ("E-1")) return 9;
	if (n.equals ("F-1")) return 10;
	if (n.equals ("G-1")) return 11;
	if (n.equals ("A-1")) return 12;
	if (n.equals ("B-1")) return 13;
	if (n.equals ("C-2")) return 14;
	return -1;
    }
}
