/**
 * This is a class with only static methods to allow for hard-coded activitiy logging.
 * Log levels are currently adjustable for:
 * <table>
 * <tr><td>0</td> <td>SILENT</td></tr>
 * <tr><td>1</td> <td>ERROR</td></tr>
 * <tr><td>2</td> <td>WARNING</td></tr>
 * <tr><td>3</td> <td>INFORMATION</td></tr>
 * <tr><td>4</td> <td>DEBUG</td></tr>
 * </table>
 * Log entries of any level can be sent at any time, but only entries with
 * a level &le; the set level will appear in the log.
 */
public class Log
{
    /** The overall log level. */
    private static int logLevel = 0;

    /** No entries at all show up in the log. */
    public static final int SILENT = 0;

    /** Only error messages will show up in the log. */
    public static final int ERROR = 1;

    /** Only error and warning messages will show up in the log. */
    public static final int WARNING = 2;

    /** Error, warning, and information messages will show up in the log. */
    public static final int INFORMATION = 3;

    /** All messages will show up in the log. */
    public static final int DEBUG = 4;

    /** Sets the overall log level. Only messages passed with a level &le;
     * this level will show up in the log.
     * @param i the desired overall log level.
     */
    public static void setLevel (int i) {logLevel = i;}

    /**
     * This writes a message to the log at the desired log level. It will be suppressed
     * if the first parameter &gt; the current overall log level.
     * @param l the log level of the message.
     * @param msg the message.
     * @see Log#setLevel (int)
     */
    public static void write (int l, String msg)
    {
	if (l > logLevel) return;
	else if (l == ERROR)
	{
	    System.out.println ("\n*** ERROR: " + msg + "\n");
	    javax.swing.JOptionPane.showMessageDialog (null, "*** ERROR: " + msg, "Log", javax.swing.JOptionPane.ERROR_MESSAGE);
	    return;
	}
	else if (l == WARNING)
	{
	    System.out.println ("\n  WARNING: " + msg + "\n");
	    java.awt.Toolkit.getDefaultToolkit ().beep ();
	    return;
	}
	else if (l == INFORMATION)
	{
	    System.out.println ("      Log: " + msg);
	    return;
	}
	else if (l == DEBUG)
	{
	    System.out.println ("    Debug [" + Thread.currentThread ().getId () + ", '" + Thread.currentThread ().getName () + "']: " + msg);
	}
    }
}
