/**
 * Rule Sets for Experiment Two. Cf. H-I, p. 158, Table 19.
 */
public class Ruleset
{
    public static final int RULE_SKIPSTEPWISE = 1;
    public static final int RULE_FORBIDTHREENOTEREPEAT = 2;
    public static final int RULE_INITIALTONIC = 3;
    public static final int RULE_CADENCE = 4;
    public static final int RULE_TRITONERESOLUTION = 5;
    public static final int RULE_OCTAVERANGE = 6;
    public static final int RULE_HARMONIC = 7;
    public static final int RULE_MELODIC = 8;
    public static final int RULE_ONE_STEPWISE = 14;
    public static final int RULE_FORBIDPARALLELPERFECTS = 9;
    public static final int RULE_FORBIDPARALLELFOURTHS = 10;
    public static final int RULE_FORBID64CHORDS = 12;
    public static final int RULE_FORBIDCLIMAXREPEAT = 13;

    private boolean skipStepwise;
    private boolean forbidThreeNoteRepeat;
    private boolean initialTonic;
    private boolean cadence;
    private boolean tritoneResolution;
    private boolean octaveRange;
    private boolean harmonic;
    private boolean melodic;
    private boolean oneStepwise;
    private boolean forbidParallelPerfects;
    private boolean forbidParallelFourths;
    private boolean forbid64Chords;
    private boolean forbidClimaxRepeat;


    public Ruleset(String ruleSet)
    {
	setRules(ruleSet);
    }
    
    
    public void setRules(String ruleSet)
    {
	if (ruleSet.toUpperCase ().equals ("A"))
	{
	    skipStepwise           = false;
	    forbidThreeNoteRepeat  = false;
	    initialTonic           = false;
	    cadence                = false;
	    tritoneResolution      = false;
	    octaveRange            = false;
	    harmonic               = false;
	    melodic                = false;
	    oneStepwise            = false;
	    forbidParallelPerfects = false;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("B"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = false;
	    cadence                = false;
	    tritoneResolution      = false;
	    octaveRange            = false;
	    harmonic               = false;
	    melodic                = false;
	    oneStepwise            = false;
	    forbidParallelPerfects = false;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("C"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = true;
	    cadence                = true;
	    tritoneResolution      = true;
	    octaveRange            = false;
	    harmonic               = false;
	    melodic                = false;
	    oneStepwise            = false;
	    forbidParallelPerfects = false;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("D"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = true;
	    cadence                = true;
	    tritoneResolution      = true;
	    octaveRange            = true;
	    harmonic               = false;
	    melodic                = false;
	    oneStepwise            = false;
	    forbidParallelPerfects = false;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("E"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = true;
	    cadence                = true;
	    tritoneResolution      = true;
	    octaveRange            = true;
	    harmonic               = true;
	    melodic                = false;
	    oneStepwise            = false;
	    forbidParallelPerfects = false;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("F"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = true;
	    cadence                = true;
	    tritoneResolution      = true;
	    octaveRange            = true;
	    harmonic               = true;
	    melodic                = true;
	    oneStepwise            = true;
	    forbidParallelPerfects = false;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("G"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = true;
	    cadence                = true;
	    tritoneResolution      = true;
	    octaveRange            = true;
	    harmonic               = true;
	    melodic                = true;
	    oneStepwise            = true;
	    forbidParallelPerfects = true;
	    forbidParallelFourths  = false;
	    forbid64Chords         = false;
	    forbidClimaxRepeat     = false;
	}
	else if (ruleSet.toUpperCase ().equals ("H"))
	{
	    skipStepwise           = true;
	    forbidThreeNoteRepeat  = true;
	    initialTonic           = true;
	    cadence                = true;
	    tritoneResolution      = true;
	    octaveRange            = true;
	    harmonic               = true;
	    melodic                = true;
	    oneStepwise            = true;
	    forbidParallelPerfects = true;
	    forbidParallelFourths  = true;
	    forbid64Chords         = true;
	    forbidClimaxRepeat     = true;
	}
	else
	    Log.write(Log.WARNING, "Unknown rule set string in Ruleset.setRules(string): '" +  ruleSet + "'!");
    }  // END setRules(String)


    public boolean getRule(int rule)
    {
	switch(rule)
	{
	    case RULE_SKIPSTEPWISE:
		return skipStepwise;
	    case RULE_FORBIDTHREENOTEREPEAT:
		return forbidThreeNoteRepeat;
	    case RULE_INITIALTONIC:
		return initialTonic;
	    case RULE_CADENCE:
		return cadence;
	    case RULE_TRITONERESOLUTION:
		return tritoneResolution;
	    case RULE_OCTAVERANGE:
		return octaveRange;
	    case RULE_HARMONIC:
		return harmonic;
	    case RULE_MELODIC:
		return melodic;
	    case RULE_ONE_STEPWISE:
		return oneStepwise;
	    case RULE_FORBIDPARALLELPERFECTS:
		return forbidParallelPerfects;
	    case RULE_FORBIDPARALLELFOURTHS:
		return forbidParallelFourths;
	    case RULE_FORBID64CHORDS:
		return forbid64Chords;
	    case RULE_FORBIDCLIMAXREPEAT:
		return forbidClimaxRepeat;
	    default:
		Log.write(Log.WARNING, "Unknown rule in Ruleset.getRule(int): " + rule + "!");
		return false;
	}
    }  // END getRule


    public static boolean getRuleFromSet(String ruleSetStr, int rule)
    {
	Ruleset ruleSet = new Ruleset(ruleSetStr);
	return ruleSet.getRule(rule);
    }
}
