import madmike.util.Log;


public class Score
{
    public static final int [] tonicTriad = {0, 2, 4, 7, 9, 11, 14};

    private int numVoices;
    private int maxLength;

    // voice index is first, chord index is second.
    private int [] [] notes;

    // one note velocity per voice, cf. H-I p. 158
    private int[] dynamics;

    private String ruleSetStr = null;
    private String info = null;

    
    public Score (int v, int l)
    {
	numVoices = v;
	maxLength = l;
	notes = new int [numVoices] [maxLength];
	dynamics = new int[numVoices];

	for(int i=0; i<numVoices; ++i)
	    dynamics[i] = 2;

	// numberGenerator = new RandomNumberGenerator ();
    }

    /*
    public Score (int v, int l, String inspClassName, String [] inspParms)
    {
	numVoices = v;
	maxLength = l;
	notes = new int [numVoices] [maxLength];
    }
    */

    public int getLength () {return maxLength;}

    public int getNumVoices () {return numVoices;}

    public void setNote (int v, int c, int n) {notes [v] [c] = n;}

    public int getNote (int v, int c) {return notes [v] [c];}

    public int[] [] getNotes() {return notes;}

    public void setDynamics(int v, int d) {dynamics[v] = d;}

    public int getDynamics(int v) {return dynamics[v];}


    public static final String [] noteLabels = {"C-0", "D-0", "E-0", "F-0", "G-0", "A-0", "B-0",
				   "C-1", "D-1", "E-1", "F-1", "G-1", "A-1", "B-1",
				   "C-2"};

    /* === BEGIN Score information setters / getters === */
    public void setRuleSetStr(String r) {ruleSetStr = r;}

    public String getRuleSetStr() {return ruleSetStr;}

    public void setInfo(String i) {info = i;}

    public String getInfo() {return info;}
    /* === END Score information setters / getters === */

    // This should only be changed in accordance with the Librarian!!
    public String toSimpleNotation ()
    {
	StringBuffer b = new StringBuffer ();
	// Write dynamics
	for (int j=numVoices - 1; j>=0; j--)
	{
	    b.append("DYN" + j + "=" + getDynamics(j) + "\n");
	}
	// Write notes
	for (int j=numVoices - 1; j>=0; j--)
	{
	    b.append ("V");
	    b.append (j);
	    b.append (": ");
	    for (int i=0; i<maxLength; i++)
	    {
		// b.append (notes [j] [i] < 10 ? " " : "");
		// b.append (notes [j] [i]);
		b.append (notes [j] [i] < 0 || notes [j] [i] > 14 ? "???" : noteLabels [notes [j] [i]]);
		b.append ("  ");
	    }
	    if (j != 0) b.append ("\n");
	}
	return b.toString ();
    }

    public String toString ()
    {
	StringBuffer b = new StringBuffer ();
	// Append dynamics
	for (int j=0; j<numVoices; ++j)
	{
	    b.append("D" + j + ":" + getDynamics(j) + (j != numVoices-1 ? "  " : ""));
	}
	b.append("\n");
	// Append notes
	for (int j=numVoices - 1; j>=0; j--)
	{
	    b.append ("V");
	    b.append (j);
	    b.append (": ");
	    for (int i=0; i<maxLength; i++)
	    {
		// b.append (notes [j] [i] < 10 ? " " : "");
		// b.append (notes [j] [i]);
		b.append (notes [j] [i] < 0 || notes [j] [i] > 14 ? "???" : noteLabels [notes [j] [i]]);
		b.append ("  ");
	    }
	    b.append ("\n");
	}
	return b.toString ();
    }
}
